<?php

namespace Drupal\comma_separated_string_widget\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\StringTextfieldWidget;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'string_textfield' widget.
 *
 * @FieldWidget(
 *   id = "comma_separated_string_textfield",
 *   label = @Translation("Textfield (comma separated values)"),
 *   field_types = {
 *     "string"
 *   },
 *   multiple_values = TRUE
 * )
 */
class CommaSeparatedStringWidget extends StringTextfieldWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    if ($items->isEmpty()) {
      return $element;
    }
    $values = array_map(static function ($item): ?string {
      return $item->value ?? NULL;
    }, (array) $items->getIterator());
    $values = array_filter($values);
    $element['value']['#default_value'] = implode(', ', $values);
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $values = parent::massageFormValues($values, $form, $form_state);
    // Explode the single value into multiple values.
    $items = array_map('trim', explode(',', $values['value']));
    $items = array_values(array_filter($items));
    return array_map(function ($item) {
      return ['value' => $item];
    }, $items);
  }

}
